<?php

namespace Lean\Api\Form;

use CSLSJ\Lean\Form\Session\EmailContext;
use Picea\Ui\Method\{ FormContextInterface, };

use Ulmus\Attribute\Property\Field;
use Lean\Api\Attribute\EntityField;

use Ulmus\Entity\EntityInterface;
use Ulmus\Entity\Field\Datetime;

abstract class Save implements \Picea\Ui\Method\FormInterface {
    use FormTrait;

    protected string $contextClass;

    public function validate(FormContextInterface $context) : bool
    {
        # Context is validating inputs
        return $context->valid($this->getEntity()->isLoaded() ? $this->getEntity() : null);
    }

    public function execute(FormContextInterface $context) : void
    {
        $entity = $this->getEntity();

        if  ($entity->isLoaded() ) {
            if (property_exists($entity, 'updatedAt')) {
                $entity->updatedAt = new Datetime();
            }
        }
        else {
            if (property_exists($entity, 'createdAt')) {
                $entity->createdAt = new Datetime();
            }
        }

        try {
            $this->assignContextToEntity($context);

            if ( $entity::repository()->save($entity) ) {
                $context->pushMessage($this->message::generateSuccess(
                    $this->lang('lean.api.form.save.success.entity')
                ));
            }
            else {
                throw new \InvalidArgumentException($this->lang('lean.api.form.save.error.entity'));
            }
        }
        catch(\Throwable $ex) {
            throw new \ErrorException($this->lang('lean.api.form.save.error.pdo', [ 'error' => $ex->getMessage() ]));
        }
    }

    protected function assignContextToEntity(FormContextInterface $context) : void
    {
        $entity = $this->getEntity();

        foreach($entity::resolveEntity()->fieldList() as $key => $property) {
            $field = $property->getAttribute(Field::class)->object;

            if (! $field->readonly || ! $entity->isLoaded()) {
                $apiField = $property->getAttribute(EntityField::class)->object ?? null;

                if ($apiField) {
                    $var = $apiField->field ?: $key;

                    if ( isset($context->{$var}) ) {
                        if ($apiField->setterMethod) {
                            # Use a setter method
                            call_user_func([ $entity, $apiField->setterMethod ], $context->{$var});
                        }
                        else {
                            # Direct property set
                            $entity->$key = $context->{$var};
                        }
                    }
                }
            }
        }
    }
}
